@@ -3,7 +3,6 @@ import random
 
 import nlp2
 import torch
-from torch import nn
 from tqdm import tqdm
 from transformers import AdamW, BertTokenizer, AutoTokenizer, AutoModel
 import numpy as np
@@ -17,7 +16,8 @@ import classifier
 import tag
 import qa
 
-from tfkit.utility import get_topP_unk_token
+from tfkit.utility import get_freqK_unk_token
+from tfkit.utility import BalancedDataParallel
 
 input_arg = {}
 
@@ -37,7 +37,7 @@ def model_train(models_list, train_dataset, models_tag, input_arg, epoch, writer
     optims = []
     models = []
     for i, m in enumerate(models_list):
-        model = nn.DataParallel(m)
+        model = BalancedDataParallel(input_arg.batch, m)
         model.train()
         models.append(model)
         optims.append(optimizer(m, input_arg.lr[i] if i < len(input_arg.lr) else input_arg.lr[0]))
@@ -58,6 +58,7 @@ def model_train(models_list, train_dataset, models_tag, input_arg, epoch, writer
                 loss.mean().backward()
                 if (total_iter + 1) % input_arg.grad_accum == 0:
                     optim.step()
+                    optim.zero_grad()
                     model.zero_grad()
                 t_loss += loss.mean().item()
                 if input_arg.tensorboard:
@@ -126,6 +127,7 @@ def _load_model_and_data(pretrained_config, tokenizer, pretrained, device):
     for model_type, train_file, test_file in zip_longest(input_arg.model, input_arg.train, input_arg.test,
                                                          fillvalue=""):
         model_type = model_type.lower()
+
         if "once" in model_type:
             train_ds = gen_once.loadOnceDataset(train_file, pretrained=pretrained_config, maxlen=input_arg.maxlen,
                                                 cache=input_arg.cache)
@@ -133,13 +135,19 @@ def _load_model_and_data(pretrained_config, tokenizer, pretrained, device):
                                                cache=input_arg.cache)
             model = gen_once.Once(tokenizer, pretrained, maxlen=input_arg.maxlen)
         elif "onebyone" in model_type:
-            train_ds = gen_onebyone.loadOneByOneDataset(train_file, pretrained=pretrained_config,
-                                                        maxlen=input_arg.maxlen,
-                                                        cache=input_arg.cache,
-                                                        likelihood=model_type)
-            test_ds = gen_onebyone.loadOneByOneDataset(test_file, pretrained=pretrained_config, maxlen=input_arg.maxlen,
-                                                       cache=input_arg.cache)
-            model = gen_onebyone.OneByOne(tokenizer, pretrained, maxlen=input_arg.maxlen, lossdrop=input_arg.lossdrop)
+            panel = nlp2.Panel()
+            inputted_arg = {"pretrained_config": pretrained_config, "maxlen": input_arg.maxlen,
+                            "cache": input_arg.cache, "likelihood": model_type, "lossdrop": input_arg.lossdrop}
+            all_arg = nlp2.function_get_all_arg_with_value(gen_onebyone.loadOneByOneDataset)
+            if input_arg.enable_arg_panel:
+                for missarg in nlp2.function_check_missing_arg(gen_onebyone.loadOneByOneDataset,
+                                                               inputted_arg):
+                    panel.add_element(k=missarg, v=all_arg[missarg], msg=missarg, default=all_arg[missarg])
+                filled_arg = panel.get_result_dict()
+                inputted_arg.update(filled_arg)
+            train_ds = gen_onebyone.loadOneByOneDataset(train_file, **inputted_arg)
+            test_ds = gen_onebyone.loadOneByOneDataset(test_file, **inputted_arg)
+            model = gen_onebyone.OneByOne(tokenizer, pretrained, **inputted_arg)
         elif 'clas' in model_type:
             train_ds = classifier.loadClassifierDataset(train_file, pretrained=pretrained_config, cache=input_arg.cache)
             test_ds = classifier.loadClassifierDataset(test_file, pretrained=pretrained_config, cache=input_arg.cache)
@@ -178,7 +186,7 @@ def main():
     parser.add_argument("--maxlen", type=int, default=368, help="max tokenized sequence length, default 368")
     parser.add_argument("--savedir", type=str, default="checkpoints/", help="model saving dir, default /checkpoints")
     parser.add_argument("--add_tokens", type=int, default=0,
-                        help="auto add top x percent UNK token to word table, range 0-100")
+                        help="auto add freq > x UNK token to word table")
     parser.add_argument("--train", type=str, nargs='+', required=True, help="train dataset path")
     parser.add_argument("--test", type=str, nargs='+', required=True, help="test dataset path")
     parser.add_argument("--model", type=str, required=True, nargs='+',
@@ -194,6 +202,7 @@ def main():
     parser.add_argument('--tensorboard', dest='tensorboard', action='store_true', help='Turn on tensorboard graphing')
     parser.add_argument("--resume", help='resume training')
     parser.add_argument("--cache", action='store_true', help='cache training data')
+    parser.add_argument("--enable_arg_panel", action='store_true', help="enable panel to input argument")
     global input_arg
     input_arg = parser.parse_args()
 
@@ -216,7 +225,7 @@ def main():
     # handling add tokens
     if input_arg.add_tokens:
         write_log("Calculating Unknown Token")
-        add_tokens = get_topP_unk_token(tokenizer, input_arg.train + input_arg.test, input_arg.add_tokens)
+        add_tokens = get_freqK_unk_token(tokenizer, input_arg.train + input_arg.test, input_arg.add_tokens)
         num_added_toks = tokenizer.add_tokens(add_tokens)
         write_log('We have added', num_added_toks, 'tokens')
         pretrained.resize_token_embeddings(len(tokenizer))
@@ -261,6 +270,7 @@ def main():
                 tag_ind = package['tags'].index(model_tag)
                 models[tag_ind].load_state_dict(state_dict)
         start_epoch = int(package.get('epoch', 1)) + 1
+
     set_seed(input_arg.seed)
     write_log("training batch : " + str(input_arg.batch * input_arg.grad_accum))
     for epoch in range(start_epoch, start_epoch + input_arg.epoch):
